<?php 

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use \Doctrine\ORM\EntityManager;


class CancelamentoServico {
    
	protected $entityManager; 

	public function __construct(EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}

    private function setReportParameters(&$params, &$conditions) {
        $params = array(
            ':DATAINI' => isset($conditions['DATAINI']) ? $conditions['DATAINI'] : null ,
            ':DATAFIN' => isset($conditions['DATAFIN']) ? $conditions['DATAFIN'] : null ,
            // ':CDFILIAL' => isset($conditions['CDFILIAL']) ? $conditions['CDFILIAL'] : null ,
            // ':CDLOJA' => isset($conditions['CDLOJA']) ? $conditions['CDLOJA'] : null ,
            ':TCDLOJA' => isset($conditions['TCDLOJA']) ? $conditions['TCDLOJA'] : null ,
            // ':CDVENDEDOR' => isset($conditions['CDVENDEDOR']) ? $conditions['CDVENDEDOR'] : null ,
            ':TCDVENDEDOR' => (isset($conditions['TCDVENDEDOR']) ? $conditions['TCDVENDEDOR'] : (isset($conditions['CDVENDEDOR']) ? ($conditions['CDVENDEDOR'] == null ? 'T' : 'F') : 'T')),
            ':NRORG' => $conditions['NRORG']
        );
    }

    private function sanitizeFilter($param) {

        $property = preg_replace('/(^_+|_+$)/', '', $param);
        $property = preg_replace('/_+/', "', '", $property);
        $property = "'" . $property . "'";

        return $property;
    }
    
    public function getCancelamentoServico($conditions) {
    	$params = [];
        $this->setReportParameters($params, $conditions);
        $CDFILIAL = $this->sanitizeFilter($conditions['CDFILIAL']);
        $CDLOJA = $this->sanitizeFilter(isset($conditions['CDLOJA']) ? $conditions['CDLOJA'] : "");
        $CDVENDEDOR = $this->sanitizeFilter(isset($conditions['CDVENDEDOR']) ? $conditions['CDVENDEDOR'] : "");

        $REL_CANCELAMENTO_SERVICO_ACRESCIMO = "
            SELECT CDLOJA, LABELFILIAL, CDCAIXA, NRSEQVENDA, DTENTRVENDA, (VRGORJETA/VRTOTPROD) PERCENT, OPERADOR, GARCON, NRMESA, CDSUPERVISOR, 
               VRTOTPROD, VRGORJETA, (VRGORJETA_ESPERADO-VRGORJETA) AS DESCGORJ, NRINSJURVEN, NRORG
            FROM (
                SELECT TABFIM.CDFILIAL||' - '||FI.NMFILIAL AS LABELFILIAL, TABFIM.CDLOJA, TABFIM.NRSEQVENDA, TABFIM.VRLIQUIDO AS VRTOTPROD, TABFIM.VRGORJETA, TABFIM.CDCAIXA,
                    TRUNC(((TABFIM.VRLIQUIDO)*(TABFIM.VRCOMISVENDA/100)), 2) AS VRGORJETA_ESPERADO, TABFIM.VRACRESCIMO, TABFIM.VRDESCONTO, TABFIM.GORJETA, TABFIM.NRCOMANDAVND,
                    CASE 
                        WHEN TABFIM.CDOPERULTATU IS NULL 
                            THEN (
                                SELECT L.CDOPERCAIXA || ' - ' || L.NMOPERCAIXA AS CDOPERULTATU
                                    FROM LOGOPERFOS L 
                                   INNER JOIN VENDA VN ON VN.CDFILIAL = L.CDFILIAL
                                                      AND VN.CDCAIXA = L.CDCAIXA
                                                      AND VN.NRMESA = SUBSTR(L.DSLIVREFOS, INSTR(L.DSLIVREFOS, ':') + 2, 4)
                                                      AND TRIM(REPLACE(VN.NRCOMANDAVND, '_', ' ')) = SUBSTR(L.DSLIVREFOS, INSTR(L.DSLIVREFOS, '/0') + 1, 10)
                                   WHERE L.IDOPERFOS = 'RET_TAX'
                                     AND VN.NRSEQVENDA = TABFIM.NRSEQVENDA
                                     AND VN.CDFILIAL = TABFIM.CDFILIAL
                                     AND VN.CDCAIXA = TABFIM.CDCAIXA
                                     AND ROWNUM = 1
                            ) 
                        ELSE TABFIM.CDOPERULTATU||' - '||OP.NMOPERADOR
                    END AS OPERADOR,
                    CASE 
                        WHEN TABFIM.CDOPERULTATU IS NULL 
                            THEN (
                                SELECT L.CDOPERRESPOP||' - '||L.NMOPERRESPOP AS CDSUPERVISOR
                                    FROM LOGOPERFOS L 
                                   INNER JOIN VENDA VN ON VN.CDFILIAL = L.CDFILIAL
                                                      AND VN.CDCAIXA = L.CDCAIXA
                                                      AND VN.NRMESA = SUBSTR(L.DSLIVREFOS, INSTR(L.DSLIVREFOS, ':') + 2, 4)
                                                      AND TRIM(REPLACE(VN.NRCOMANDAVND, '_', ' ')) = SUBSTR(L.DSLIVREFOS, INSTR(L.DSLIVREFOS, '/0') + 1, 10)
                                   WHERE L.IDOPERFOS = 'RET_TAX'
                                     AND VN.NRSEQVENDA = TABFIM.NRSEQVENDA
                                     AND VN.CDFILIAL = TABFIM.CDFILIAL
                                     AND VN.CDCAIXA = TABFIM.CDCAIXA
                                     AND ROWNUM = 1
                            )
                        ELSE 'Retirada sem supervisor'
                    END AS CDSUPERVISOR,
                    DECODE (TABFIM.CDVENDEDOR ||' - '||VE.NMRAZSOCVEN, ' ', 'Venda Sem Garçon', TABFIM.CDVENDEDOR ||' - '||VE.NMRAZSOCVEN) AS GARCON,
                    TABFIM.VRCOMISVENDA, TABFIM.DTENTRVENDA, TABFIM.NRMESA||' - N° Venda: '||TABFIM.NRSEQVENDA AS NRMESA, VE.NRINSJURVEN, TABFIM.NRORG
                FROM (
                    SELECT TAB3.CDFILIAL, TAB3.CDLOJA, TAB3.NRSEQVENDA, SUM(TAB3.VRACRESCIMO) AS VRACRESCIMO, SUM(TAB3.VRDESCONTO) AS VRDESCONTO, SUM(TAB3.VRGORJETA) AS GORJETA, TAB3.NRORG,
                    SUM(TAB3.VRACRESCIMO+TAB3.VRGORJETA) AS VRGORJETA, SUM(TAB3.VALOR_LIQUIDO) AS VRLIQUIDO, L.VRCOMISVENDA, TAB3.NRCOMANDAVND, TAB3.CDCAIXA, TAB3.NRMESA, TAB3.CDOPERULTATU, TAB3.CDVENDEDOR, TAB3.DTENTRVENDA
                        FROM (  
                            SELECT TAB1.CDFILIAL, TAB1.CDLOJA, TAB1.NRSEQVENDA, 
                                TRUNC(SUM(NVL(TAB1.VALOR_LIQUIDO, 0)), 2) AS VALOR_LIQUIDO,
                                SUM(NVL(TAB1.VRACRESCIMO, 0)) AS VRACRESCIMO,
                                SUM(NVL(TAB1.VRDESCONTO, 0)) AS VRDESCONTO,
                                SUM(NVL(TAB2.VALOR_TOTAL, 0) - NVL(TAB2.VALOR_DESC, 0)) AS VRGORJETA,
                                SUM(NVL(TAB2.VALOR_DESC, 0)) AS DESC_VRGORJETA,
                                SUM(NVL(TAB1.VRCANCELAMENTO, 0)) AS VRCANCELAMENTO,
                                SUM(NVL(TAB1.VRCANCBRUTO, 0)) AS VRCANCBRUTO,	 
                                SUM(NVL(TAB1.QTD_TOTAL, 0) + NVL(TAB2.QTD_TOTAL, 0)) AS QTD_TOTAL,
                                TAB1.NRCOMANDAVND, TAB1.CDCAIXA, TAB1.NRMESA,
                                TAB1.CDOPERULTATU, TAB1.CDVENDEDOR, TAB1.DTENTRVENDA, TAB1.NRORG
                                    FROM (
                                        SELECT CDFILIAL ,NMFILIAL ,MODALIDADE ,CDCAIXA ,DTENTRVENDA, NRSEQVENDA, CDLOJA, NRCOMANDAVND, NRMESA, CDVENDEDOR, NRORG,
                                            SUM(VRDESCONTO) AS VRDESCONTO ,SUM(VRACRESCIMO) AS VRACRESCIMO ,SUM(VALOR_LIQUIDO) AS VALOR_LIQUIDO ,SUM(QTD_TOTAL) AS QTD_TOTAL ,0 AS VRCANCELAMENTO ,0 AS VRCANCBRUTO, CDOPERULTATU
                                        FROM (
                                            WITH FLC AS (
                                                    SELECT DISTINCT V.CDFILIAL, V.CDLOJA ,V.CDCAIXA ,L.IDCOMISVENDA ,V.NMFILIAL ,L.CDPRODTAXAENTR ,L.IDTRATTAXAENTR ,L.CDPRODTAXASERV ,L.IDTRATTAXASERV ,L.CDPRODTXEXCEDE
                                                    FROM V_FILIALLOJACAIXA V,LOJA L
                                                    WHERE ((:TCDLOJA = 'T') OR (V.CDFILIAL || V.CDLOJA IN (".$CDLOJA.")) OR (V.CDLOJA IN (".$CDLOJA.")))
                                                        AND INSTR(".$CDFILIAL.", V.CDFILIAL) > 0
                                                        AND V.CDFILIAL = L.CDFILIAL
                                                        AND V.CDLOJA = L.CDLOJA
                                                    )
                                            SELECT ITV.CDFILIAL,ITV.CDCAIXA,ITV.NRSEQVENDA,V.DTENTRVENDA, NVL(O.DSORIGEMVENDA, 'Modalidade Desconhecida') AS MODALIDADE,FLC.NMFILIAL, V.NRCOMANDAVND, NRMESA,
                                                V.CDOPERULTATU, V.CDVENDEDOR, V.NRORG,
                                                CASE V.IDSITUVENDA
                                                    WHEN 'C'
                                                        THEN 0
                                                    ELSE SUM(TRUNC(NVL(ITV.VRDESITVEND, 0), 2))
                                                    END AS VRDESCONTO,
                                                (CASE 
                                                    WHEN FLC.CDPRODTXEXCEDE = ITV.CDPRODUTO
                                                        AND ITV.IDPRODTAXASERV IN (
                                                            'S'
                                                            ,'X'
                                                            )
                                                        THEN SUM(TRUNC(ITV.QTPRODVEND * (ITV.VRUNITVEND + NVL(ITV.VRUNITVENDCL, 0)), 2) + ITV.VRACRITVEND - ITV.VRDESITVEND)
                                                    WHEN V.IDSITUVENDA = 'C'
                                                        THEN 0
                                                    ELSE SUM(TRUNC(NVL(ITV.VRACRITVEND, 0), 2))
                                                END) AS VRACRESCIMO,
                                                (CASE 
                                                    WHEN (FLC.CDPRODTXEXCEDE = ITV.CDPRODUTO AND ITV.IDPRODTAXASERV IN ('S','X')) OR V.IDSITUVENDA = 'C'
                                                        THEN 0
                                                    ELSE SUM(TRUNC(ITV.QTPRODVEND * (ITV.VRUNITVEND + NVL(ITV.VRUNITVENDCL, 0)) + ITV.VRACRITVEND - ITV.VRDESITVEND, 2))
                                                END) AS VALOR_LIQUIDO,
                                                SUM(ITV.QTPRODVEND) AS QTD_TOTAL,
                                                V.CDLOJA
                                            FROM VENDA V
                                            INNER JOIN FLC ON V.CDFILIAL = FLC.CDFILIAL
                                                AND V.CDCAIXA = FLC.CDCAIXA
                                                AND V.CDLOJA = FLC.CDLOJA
                                            INNER JOIN ITEMVENDA ITV ON V.CDFILIAL = ITV.CDFILIAL
                                                AND V.CDCAIXA = ITV.CDCAIXA
                                                AND V.NRSEQVENDA = ITV.NRSEQVENDA
                                            LEFT JOIN ORIGEMVENDAFOS O ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                                                AND V.NRORG = O.NRORG
                                            WHERE V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
                                                AND V.IDSITUVENDA = 'O'
                                                AND ITV.CDPRODUTO <> (CASE WHEN (FLC.IDTRATTAXAENTR = 'P' AND SUBSTR(V.IDORIGEMVENDA, 0, 3) = 'DLV') THEN FLC.CDPRODTAXAENTR ELSE 'ZZZZZZZ' END)
                                                AND ITV.CDPRODUTO <> (CASE WHEN (FLC.IDTRATTAXASERV = 'P' AND FLC.IDCOMISVENDA IN ('S','G')) THEN FLC.CDPRODTAXASERV ELSE 'ZZZZZZZ' END)
                                                AND FLC.CDFILIAL IN (".$CDFILIAL.") 
                                                AND ((:TCDLOJA = 'T') OR (FLC.CDFILIAL || FLC.CDLOJA IN (".$CDLOJA.")) OR (FLC.CDLOJA IN (".$CDLOJA.")))
                                                AND ((:TCDVENDEDOR = 'T') OR (NVL(ITV.CDVENDEDOR,V.CDVENDEDOR) IN (".$CDVENDEDOR.")))	
                                            GROUP BY ITV.CDFILIAL, ITV.CDCAIXA, O.DSORIGEMVENDA, ITV.NRSEQVENDA, V.IDSITUVENDA, ITV.IDPRODTAXASERV, NRMESA, V.CDVENDEDOR,
                                                ITV.VRACRITVEND, FLC.CDPRODTXEXCEDE, V.DTENTRVENDA, FLC.NMFILIAL, ITV.CDPRODUTO, V.CDLOJA, V.NRCOMANDAVND, V.CDOPERULTATU, V.NRORG
                                            )
                                        GROUP BY CDFILIAL, NMFILIAL, CDCAIXA, MODALIDADE, DTENTRVENDA, NRSEQVENDA, CDLOJA, NRCOMANDAVND, NRMESA, CDOPERULTATU, CDVENDEDOR, NRORG
                                    ) TAB1
                                    LEFT JOIN (
                                        SELECT TB2.CDFILIAL, TB2.DTENTRVENDA, TB2.MODALIDADE, SUM(TB2.QTD_TOTAL) AS QTD_TOTAL, TRUNC(SUM(TB2.VALOR_TOTAL), 2) AS VALOR_TOTAL, 
                                            TRUNC(SUM(TB2.VALOR_DESC), 2) AS VALOR_DESC, TB2.CDLOJA, TB2.NRSEQVENDA
                                        FROM ( 
                                            SELECT V.CDFILIAL, V.DTENTRVENDA, NVL(O.DSORIGEMVENDA, 'Modalidade Desconhecida') AS MODALIDADE, 
                                                SUM(I.QTPRODVEND) AS QTD_TOTAL, 
                                                (SUM(TRUNC(NVL(I.QTPRODVEND, 0) * (NVL(I.VRUNITVEND, 0) + NVL(I.VRUNITVENDCL, 0)), 2)) + SUM(NVL(I.VRACRITVEND,0))) AS VALOR_TOTAL, 
                                                SUM(TRUNC(NVL(I.VRDESITVEND, 0), 2)) AS VALOR_DESC, V.CDLOJA, V.NRSEQVENDA
                                            FROM VENDA V
                                                JOIN (SELECT DISTINCT V.CDFILIAL, V.CDLOJA, V.CDCAIXA, L.IDCOMISVENDA, V.NMFILIAL, L.CDPRODTAXAENTR, L.IDTRATTAXAENTR, L.CDPRODTAXASERV, L.IDTRATTAXASERV, L.CDPRODTXEXCEDE
                                                         FROM V_FILIALLOJACAIXA V, LOJA L
                                                           WHERE ((:TCDLOJA = 'T') OR (v.CDFILIAL || v.CDLOJA IN (".$CDLOJA.")) OR (v.CDLOJA IN (".$CDLOJA.")))
                                                             AND INSTR(".$CDFILIAL.", V.CDFILIAL) > 0
                                                             AND V.CDFILIAL = L.CDFILIAL
                                                             AND V.CDLOJA = L.CDLOJA
                                                ) FLC
                                                    ON V.CDFILIAL = FLC.CDFILIAL
                                                        AND V.CDCAIXA = FLC.CDCAIXA
                                                        AND V.CDLOJA = FLC.CDLOJA
                                                JOIN ITEMVENDA I
                                                    ON V.NRSEQVENDA = I.NRSEQVENDA
                                                        AND V.CDCAIXA = I.CDCAIXA
                                                        AND V.CDFILIAL = I.CDFILIAL
                                                        AND I.CDPRODUTO = FLC.CDPRODTAXASERV
                                                LEFT JOIN ORIGEMVENDAFOS O
                                                    ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                                                        AND V.NRORG = O.NRORG
                                            WHERE FLC.IDCOMISVENDA IN ('S', 'G')      
                                                AND FLC.IDTRATTAXASERV = 'P'                             
                                                AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
                                                AND V.IDSITUVENDA = 'O'               
                                                AND SUBSTR(V.IDORIGEMVENDA, 0, 3) <> 'DLV'
                                                AND FLC.CDFILIAL IN (".$CDFILIAL.") 
                                                AND ((:TCDLOJA = 'T') OR (FLC.CDFILIAL || FLC.CDLOJA IN (".$CDLOJA.")) OR (FLC.CDLOJA IN (".$CDLOJA.")))
                                                AND ((:TCDVENDEDOR = 'T') OR (NVL(I.CDVENDEDOR,V.CDVENDEDOR) IN (".$CDVENDEDOR.")))		 
                                            GROUP BY V.CDFILIAL, V.DTENTRVENDA, O.DSORIGEMVENDA, V.CDLOJA, V.NRSEQVENDA
                                        ) TB2
                                        GROUP BY TB2.CDFILIAL, TB2.DTENTRVENDA, TB2.MODALIDADE, TB2.CDLOJA,TB2.NRSEQVENDA
                                    ) TAB2
                                        ON TAB1.CDFILIAL = TAB2.CDFILIAL
                                        AND TAB1.DTENTRVENDA = TAB2.DTENTRVENDA
                                        AND TAB1.MODALIDADE = TAB2.MODALIDADE
                                        AND TAB1.CDLOJA = TAB2.CDLOJA
                                        AND TAB1.NRSEQVENDA = TAB2.NRSEQVENDA
                            GROUP BY TAB1.CDFILIAL, TAB1.CDLOJA, TAB1.NRSEQVENDA, TAB1.NRCOMANDAVND, TAB1.NRMESA, TAB1.CDCAIXA, TAB1.CDOPERULTATU, TAB1.CDVENDEDOR, TAB1.DTENTRVENDA, TAB1.NRORG
                        ) TAB3
                        LEFT JOIN LOJA L
                            ON TAB3.CDLOJA = L.CDLOJA
                            AND TAB3.CDFILIAL = L.CDFILIAL
                    GROUP BY TAB3.CDFILIAL, TAB3.CDLOJA, TAB3.NRSEQVENDA, L.VRCOMISVENDA, TAB3.NRCOMANDAVND, TAB3.NRMESA,TAB3.CDCAIXA, TAB3.CDOPERULTATU, TAB3.CDVENDEDOR, TAB3.DTENTRVENDA, TAB3.NRORG
                ) TABFIM
                LEFT JOIN OPERADOR OP ON OP.CDOPERADOR = TABFIM.CDOPERULTATU 
                LEFT JOIN FILIAL FI ON TABFIM.CDFILIAL = FI.CDFILIAL
                LEFT JOIN VENDEDOR VE ON TABFIM.CDVENDEDOR = VE.CDVENDEDOR
            ) WHERE VRGORJETA_ESPERADO > VRGORJETA
        ";
        
        return $this->entityManager->getConnection()->fetchAll($REL_CANCELAMENTO_SERVICO_ACRESCIMO, $params);
    }
    
}